from dataclasses import dataclass
import dataclasses
import typing

class StigPieceMetaClass(type):
    t_piece = {}
    m_piece = {}
    b_piece = {}

    def __call__(cls, *args, **kwargs):
        if "__stig_pos__" not in kwargs:
            raise ValueError("__stig_pos__ is required")
        if "__stig_set__" not in kwargs:
            raise ValueError("__stig_set__ is required")
        stig_pos = kwargs.get("__stig_pos__", None)
        if stig_pos is None:
            raise ValueError("__stig_pos__ cannot be None")
        if stig_pos < 0 or stig_pos > 3:
            raise ValueError("__stig_pos__ must be between 0 and 3")
        
        stig_set = kwargs.get("__stig_set__", None)
        if stig_set is None or not isinstance(stig_set, StigamataSet):
            raise ValueError("__stig_set__ cannot be None")
        stig_name = stig_set.name
        stig_pos = int(stig_pos)
        
        if stig_pos == 0:
            interested_dict = cls.t_piece
        elif stig_pos == 1:
            interested_dict = cls.m_piece
        else:
            interested_dict = cls.b_piece

        if stig_name not in interested_dict:
            interested_dict[stig_name] = super().__call__(*args, **kwargs)
        
        return interested_dict[stig_name]


@dataclass
class StigamataPiece(metaclass=StigPieceMetaClass):
    __stig_pos__ : int
    __stig_set__ : 'StigamataSet'

    def __post_init__(self):
        if not (0 <= self.__stig_pos__ <=2):
            raise ValueError("__stig_pos__ must be between 0 and 2")

    @property
    def pos(self) -> int:
        return self.__stig_pos__

    @property
    def is_top(self):
        return self.__stig_pos__ == 0

    @property
    def is_middle(self):
        return self.__stig_pos__ == 1

    @property
    def is_bottom(self):
        return self.__stig_pos__ == 2

    @staticmethod
    def get_top(stig_name : str):
        return StigamataPiece.t_piece.get(stig_name, None)
    
    @staticmethod
    def get_middle(stig_name : str):
        return StigamataPiece.m_piece.get(stig_name, None)

    @staticmethod
    def get_bottom(stig_name : str):
        return StigamataPiece.b_piece.get(stig_name, None)
    
    @property
    def effect(self):
        return self.__stig_set__.effect(self.__stig_pos__)

    @property
    def hoyo_id(self):
        return self.__stig_set__._lab_ids[self.__stig_pos__]

    @property
    def stigset(self) -> 'StigamataSet':
        """
        returns the stig set object of this piece
        """
        return self.__stig_set__

    def __str__(self) -> str:
        if self.is_top:
            return f"{self.__stig_set__._set_name} (T)"
        elif self.is_middle:
            return f"{self.__stig_set__._set_name} (M)"
        else:
            return f"{self.__stig_set__._set_name} (B)"


class StigmataSetMetaClass(type):
    instances = {}
    alt_name_instances = {}

    def __call__(cls, *args, **kwargs):
        if "_set_name" not in kwargs:
            raise ValueError("_set_name is required")
        set_name = kwargs.get("_set_name", None)
        if set_name is None:
            raise ValueError("_set_name cannot be None")

        alt_names = kwargs.pop("alt_names", None)


        if set_name not in cls.instances:
            make_item = super().__call__(*args, **kwargs)
            set_name = set_name.lower()

            if alt_names and isinstance(alt_names, typing.List[str]):
                    for name in alt_names:
                        name = name.lower()
                        cls.alt_name_instances[name] = cls.instances[set_name]
            
            cls.instances[set_name] = make_item

        return cls.instances[set_name]

@dataclass
class StigamataSet(metaclass=StigmataSetMetaClass):
    _set_name : str
    _top_e : str = None
    _mid_e : str = None
    _bot_e : str = None
    _two_piece : str = None
    _three_piece : str = None
    _lab_id : int = None
    # default factory
    _lab_ids : list = dataclasses.field(default_factory=lambda : [None, None, None])
    _else : dict = dataclasses.field(default_factory=lambda : {})

    def __post_init__(self):
        top = self.top
        mid = self.middle
        bot = self.bottom

    @property
    def name(self):
        return self._set_name

    def effect(self, pos: int):
        if pos == 0:
            return self._top_e
        elif pos == 1:
            return self._mid_e
        elif pos == 2:
            return self._bot_e
        else:
            raise ValueError("pos must be between 0 and 2")

    def has_effect(self, pos: int):
        return self.effect(pos) is not None

    @property
    def has_top(self):
        return self._top_e is not None

    @property
    def has_middle(self):
        return self._mid_e is not None
    
    @property
    def has_bottom(self):
        return self._bot_e is not None

    @property
    def top(self):
        if not self.has_top:
            return None

        return StigamataPiece(
            __stig_pos__=0,
            __stig_set__=self
        )

    @property
    def middle(self):
        if not self.has_middle:
            return None

        return StigamataPiece(
            __stig_pos__=1,
            __stig_set__=self
        )
    
    @property
    def bottom(self):
        if not self.has_bottom:
            return None
    
        return StigamataPiece(
            __stig_pos__=2,
            __stig_set__=self
        )

    @property
    def two_piece(self) -> str:
        return self._two_piece
    
    @property
    def three_piece(self) -> str:
        return self._three_piece

    @staticmethod
    def create(
        name : str, 
        top : str = None, 
        mid : str = None, 
        bot : str = None, 
        two_piece : str = None, 
        three_piece : str = None,
        top_id : int = None,
        mid_id : int = None,
        bot_id : int = None,
        id : int = None,
        alternative_names : typing.List[str] = None,
        **kwargs
    ):
        return StigamataSet(
            _set_name=name,
            _top_e=top,
            _mid_e=mid,
            _bot_e=bot,
            _two_piece=two_piece,
            _three_piece=three_piece,
            _lab_id=id,
            _lab_ids=[top_id, mid_id, bot_id],
            alt_names = alternative_names,
            _else=kwargs
        )

    @staticmethod
    def iterate() -> typing.Generator['StigamataSet', None, None]:
        for val in StigmataSetMetaClass.instances.values():
            yield val

    @staticmethod
    def get(name : str, alt : bool = False, partial_search : bool = False) -> typing.Union['StigamataSet', None]:
        name = name.lower()
        if alt and name in StigmataSetMetaClass.alt_name_instances:
            return StigmataSetMetaClass.alt_name_instances[name]

        getfull = StigmataSetMetaClass.instances.get(name, None)
        if getfull is not None:
            return getfull
        
        if partial_search:
            for val in StigamataSet.iterate():
                val : StigamataSet
                if val.name.lower().startswith(name):
                    return val


from honkaiDex.base.stigamata import StigamataSet
import logging
import json
from honkaiDex.data import HONKAIDEX_DATA, load_once
from honkaiDex import config
import os 

@load_once()
def load():

    logging.debug(f"Loading {config.data.stigamata_1.FILENAME}")

    json_data = {}

    with open(os.path.join(HONKAIDEX_DATA, config.data.stigamata_1.FILENAME), "r") as f:
        json_data = json.load(f)

    for item in json_data:
        StigamataSet.create(
            name=item["name"],
            top=item["top_e"],
            mid=item["mid_e"],
            bot=item["bot_e"],
            two_piece=item["two_piece"],
            three_piece=item["three_piece"],
            obtainable=item["obtainable"],
        )
    



from dataclasses import dataclass
import dataclasses
import typing


class BaseCharacterMeta(type):
    instances = {}
    nicknames : dict = {}
    def __call__(cls, *args, **kwargs):
        nicknames = kwargs.get("_nicknames", None)
        name = kwargs.get("_name", None)
        
        if name is None:
            raise ValueError("_name is required")
        name : str = name.lower().strip()
        
        if nicknames is None:
            nicknames = []

        if cls not in cls.instances:
            kwargs.pop("_nicknames", None)
            kwargs.pop("_name", None)
            cls.instances[name] = super().__call__(_nicknames=nicknames, _name=name, _else=kwargs)
            for nickname in nicknames:
                cls.nicknames[nickname] = cls.instances[name]
        
        return cls.instances[name]

@dataclass
class BaseCharacter(metaclass=BaseCharacterMeta):
    _name : str
    _nicknames : typing.List[str] = dataclasses.field(default_factory=lambda : [])
    _else : dict = dataclasses.field(default_factory=lambda : {})
    @property
    def name(self):
        return self._name

    @property
    def nicknames(self):
        return self._nicknames

    @staticmethod
    def get(name: str, nickname_allowed : bool = False, partial_search : bool = False) -> typing.Union['BaseCharacter', None]:
        name = name.strip().lower()

        item = BaseCharacter.instances.get(name, None)
        if item is not None:
            return item

        if nickname_allowed:
            item = BaseCharacter.nicknames.get(name, None)
            if item is not None:
                return item

        if partial_search:
            for val in BaseCharacter.instances.values():
                if name in val.name:
                    return val
        
        if nickname_allowed and partial_search:
            for key, val in BaseCharacter.nicknames.items():
                if name in key:
                    return val

                

    def __del__(self):
        self.__class__.instances.pop(self.name, None)
        for nickname in self.nicknames:
            self.__class__.nicknames.pop(nickname, None)

    @staticmethod
    def create(name: str, nicknames: typing.List[str] = None, **kwargs):
        if nicknames is None:
            nicknames = []
        return BaseCharacter(name=name, _nicknames=nicknames, **kwargs)


from enum import Enum
class BattlesuitType(Enum):
    MECH = "Mech"
    PSY = "Psy"
    BIO = "Bio"
    QUA = "Qua"
    IMG = "Img"

class BattlesuitMeta(type):
    abbrevs = {}
    instances = {}
    def __call__(cls, *args, **kwargs):
        name = kwargs.get("_name", None)

        if name is None:
            raise ValueError("name is required")
        
        name : str = name.lower().strip()

        abbrevs = kwargs.get("_battlesuit_abbrevs", [])
        if abbrevs is None:
            abbrevs = []
        

        if name not in cls.instances:
            cls.instances[name] = super().__call__(*args, **kwargs)
            for abbrev in abbrevs:
                abbrev = abbrev.lower().strip()
                cls.abbrevs[abbrev] = name


        return cls.instances[name]

@dataclass
class Battlesuit(metaclass=BattlesuitMeta):
    _base_character : BaseCharacter
    _name : str
    _version_released : str
    _rairty : str
    _tags : typing.List[str] = dataclasses.field(default_factory=lambda : [])
    _img_link : str = None
    _else : dict = dataclasses.field(default_factory=lambda : {})
    _type : BattlesuitType = None
    _battlesuit_abbrevs : typing.List[str] = dataclasses.field(default_factory=lambda : [])

    @property
    def bs_type(self):
        return self._type

    @property
    def character(self):
        return self._base_character

    @property
    def shards_needed(self):
        return self._unlock_shards

    @property
    def name(self):
        return self._name
    
    @property
    def version_released(self):
        return self._version_released

    @property
    def rarity(self):
        return self._rairty

    @property
    def tags(self):
        return self._tags

    @property
    def img_link(self):
        return self._img_link
    
    @property
    def meta(self):
        return self._else

    def __str__(self):
        return f"{self.name} ({self.bs_type})"

    @staticmethod
    def all():
        return list(Battlesuit.instances.values())
    
    def __repr__(self) -> str:
        return self.__str__()

    @staticmethod
    def get(name: str , allow_nickname : bool = False, partial_search : bool = False):
        name = name.strip().lower()
        item = Battlesuit.instances.get(name, None)
        if item is not None:
            return item

        if allow_nickname:
            name = Battlesuit.abbrevs.get(name, None)
            if name is not None:
                return Battlesuit.instances[name]

        if partial_search:
            for item in Battlesuit.instances.values():
                item : Battlesuit
                if name in item.name.lower():
                    return item
        
        if allow_nickname and partial_search:
            for val in Battlesuit.abbrevs.keys():
                if name in val:
                    return Battlesuit.instances[Battlesuit.abbrevs[name]]

    @staticmethod
    def get_all(**kwargs):
        ret = []
        for item in Battlesuit.instances.values():
            item : Battlesuit
            if all(kwargs[key] == getattr(item, "_"+key) for key in kwargs.keys()):
                ret.append(item)
        return ret

    @staticmethod
    def create(
        name: str, 
        base_character: BaseCharacter, 
        type_   : str,
        version_released: str, 
        rarity: str,
        tags: typing.List[str] = None, 
        img_link: str = None, 
        abbrevs: typing.List[str] = None,
        **kwargs
    ):
        if tags is None:
            tags = []
        # get battlesuit_type
        for bs_type in BattlesuitType:
            if bs_type.value == type_:
                type_ = bs_type
                break
        

        return Battlesuit(
            name=name, 
            _base_character=base_character, 
            _version_released=version_released, 
            _rairty=rarity, 
            _type=type_,
            _tags=tags, 
            _img_link=img_link, 
            _battlesuit_abbrevs=abbrevs,
            _else=kwargs
        )


from zxutil.collections import _gconfig, _cconfig

class gconfig(_gconfig):
    MIN_LV = 1
    MAX_LV = 88

    STIGAMATA = "stigamata_1"
    CHARACTER = "character_0"
class config(_cconfig):    
    class data(_cconfig):
        class stigamata_1(_cconfig):
            FILENAME = "stigamata_1.json"

        class character_0(_cconfig):
            FILENAME = "character_0.json"

    class profile(_cconfig):
        class cached(_cconfig):
            STIGAMATA = gconfig
            CHARACTER = gconfig
        class just_stigamata(_cconfig):
            STIGAMATA = gconfig

        class just_character(_cconfig):
            CHARACTER = gconfig
            


import json
import logging
import os
from time import sleep
from bs4 import BeautifulSoup
import requests

# import parent dir 
import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

BASE_URL = "https://honkaiimpact3.fandom.com"
URL = "https://honkaiimpact3.fandom.com/wiki/Stigmata"


from honkaiDex.data import HONKAIDEX_DATA

# get script filename
script_name = os.path.basename(__file__).split(".")[0]

data = []

def parse_stig_page(href : str, name : str, top_img=None, mid_img =None, bot_img =None):
    page = requests.get(BASE_URL + href)
    soup = BeautifulSoup(page.content, 'html.parser')

    # 
    entry = {}

    # 
    if "(" in name:
        name = name.split("(")[0]
    entry["name"] = name

    # get effects div
    base_effects_div = soup.find("div" ,id="mw-content-text")
    
    effect_divs = base_effects_div.find_all("div", style="width: fit-content; max-width: 600px; padding: 5px; display: flex; flex-direction: column; margin: 2px; box-shadow: 0 0 2px rgb(0 0 0 / 50%); border-radius: 5px;")
    for i, effect in enumerate(effect_divs):
        # get 2nd div in effect div
        effect_div = effect.find_all("div")[1]
        # get str
        effect_str = effect_div.get_text()
        
        if i == 0:
            entry["top_e"] = effect_str
        elif i == 1:
            entry["mid_e"] = effect_str
        elif i == 2:
            entry["bot_e"] = effect_str
    
    binded_effects_div = base_effects_div.find("div", style="display: flex; flex-wrap: wrap; gap: 10px;", class_="cflex-nowrap")
    binded_effects_div = binded_effects_div.find("div", style="display: flex; flex-direction: column;")
    binded_effects_divs = binded_effects_div.find_all("div")
    for i, binded_effect in enumerate(binded_effects_divs):
        if i == 1:
            entry["two_piece"] = binded_effect.get_text()
        elif i == 3:
            entry["three_piece"] = binded_effect.get_text()
    
    # image
    if top_img is not None:
        entry["top_img"] = top_img

    if mid_img is not None:
        entry["mid_img"] = mid_img
    
    if bot_img is not None:
        entry["bot_img"] = bot_img

    # get obtainable from data
    try:
        obtainable_div = base_effects_div.find("div", style="min-width: fit-content; width: fit-content; padding: 5px; display: flex; flex-direction: column; gap: 5px; margin: 2px; box-shadow: 0 0 2px rgb(0 0 0 / 50%); border-radius: 5px;", class_="infobox-base")
        # find all divs inside
        obtainable_divs = obtainable_div.find_all("div")
        entry["obtainable"] = []
        for i, odive in enumerate(obtainable_divs):
            if i == 0:
                continue
            # get text
            
            obtainable_str = odive.get_text()
            if "❌" in obtainable_str:
                continue

            obtainable_str = obtainable_str[2:]
            entry["obtainable"].append(obtainable_str)
    except:
        pass
    
    global data
    data.append(entry)

def parse(path:str):
    page = requests.get(URL)
    soup = BeautifulSoup(page.content, 'html.parser')
    # get div main-container 
    main_container = soup.find("div", {"class" : "main-container"})

    parent_div = main_container.find("div", {"style" : "display: flex; flex-wrap: wrap; gap: 5px;"})

    # get all divs
    divs = parent_div.find_all("div", style="border-radius: 10px; padding: 2px; background-color: rgb(55 87 126 / 20%); border: 1px solid #2E85B5; width: min-content; display: flex; word-break: break-word; text-align: center; font-size: 13px; flex-direction: column;")

    for div in divs:
        stig_meta = div.find_all("a")
        stig_div = stig_meta[0]

        alt_image = stig_div.find("img")
        print(f"found {stig_div.get('href')}")
        logging.debug(alt_image['data-src'])

        parse_stig_page(
            href=stig_div.get('href'),
            name= stig_div.get('title'),  
            top_img=alt_image['data-src'], 
            mid_img=stig_meta[1].find("img")['data-src'],
            bot_img=stig_meta[2].find("img")['data-src'],
        )
        sleep(1)

    # write to file
    with open(path, "w") as f:
        json.dump(data, f, indent=4)

if __name__ == '__main__':
    parse(os.path.join(HONKAIDEX_DATA, f"{script_name}.json"))


from honkaiDex.base.character import BaseCharacter, Battlesuit
import logging
import json
from honkaiDex.data import HONKAIDEX_DATA, load_once
from honkaiDex import config
import os 

@load_once()
def load():

    logging.debug(f"Loading {config.data.character_0.FILENAME}")

    json_data :dict = {}

    with open(os.path.join(HONKAIDEX_DATA, config.data.character_0.FILENAME), "r") as f:
        json_data = json.load(f)

    for character_data in json_data.values():
        character_data : dict
        character = BaseCharacter(
            _name=character_data["name"],
            _nicknames=character_data.get("nicknames", []),
        )


        for bs_data in character_data.get("battlesuit", []):
            bs_data : dict
            bs = Battlesuit(
                _base_character=character,
                _name=bs_data["name"],
                _type=bs_data["type"],
                _version_released=bs_data["version"],
                _rairty=bs_data["rank"],
                _tags=bs_data.get("tags", []),
                _img_link=bs_data.get("img_link", None),
                _else=bs_data.get("else", {}),
            )



import json
import logging
import os
from time import sleep
from bs4 import BeautifulSoup
import requests
import bs4

# import parent dir 
import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

URL = "https://honkaiimpact3.fandom.com/wiki/Battlesuits"
BASE_URL= "https://honkaiimpact3.fandom.com"
from honkaiDex.data import HONKAIDEX_DATA

# get script filename
script_name = os.path.basename(__file__).split(".")[0]

data = {}


def parse_base_character(td : bs4.element.Tag):
    # get a
    a = td.find("a")

    href = a.get("href")
    name = a.get("title").strip().lower()

    logging.debug(f"found : {name}")
    dict_data = {
        "link" : href,
    }

    data[name] = dict_data
    return name


def get_bs_rank(main_class : bs4.element.Tag):
    rank_span = main_class.find("span", class_="nomobile")
    rank_img = rank_span.find("img")
    rank = rank_img.get("alt")[0]
    logging.debug(f"rank: {rank}")
    return rank

def get_bs_img(main_class : bs4.element.Tag):
    div = main_class.find("div", class_="stretch-image",
        style="position: relative; max-width: 150px; padding: 1px;"
    )
    img = div.find("img")
    alt = img.get("alt")
    src = img.get("src")
    logging.debug(f"battlesuit: {alt}")
    return alt, src

def get_bs_type(profile_box : bs4.element.Tag):
    # get type
    colors = ["#39d5fd", "#9976ff", "#fd44ac", "#feb22b","#eed390"]

    for color in colors:
        type_span = profile_box.find("span",
            style=f"display: flex; gap: 5px; color: {color};text-align:center;"
        )
        if type_span is not None:
            break

    if type_span is None:
        logging.error(f"not found type")
        return None

    type_span = type_span.find("img")
    return type_span.get("alt")

def get_bs_release_version(profile_box : bs4.element.Tag):
    # get type
    release_div = profile_box.find("div",
        style="border-top: 2px solid rgb(255 255 255 / 20%); line-height: 160%;"
    )

    links = release_div.find_all("a")

    for link in links:
        if link.get("title").startswith("Version"):
            return str(link.get("title").split(" ")[1])

def get_bs_core_strengths(profile_box : bs4.element.Tag):
    attrs = []
    div = profile_box.find("div",
        style="line-height: 160%; padding-right: 5px; margin-left: 10px; margin-right: 10px; padding-bottom: 5px;"
    )
    div = div.find("div",
        style="display: flex; flex-wrap: wrap; gap: 5px;",
    )

    divs = div.find_all("div",
        style="background: white; width: fit-content; border-radius: 5px; box-shadow: inset 0 0 1px 0px #2e85b5; display: flex; align-items: center; padding: 3px; gap: 3px;"
    )
    
    for div in divs:
        div : bs4.element.Tag
        two_divs = div.find_all("div")
        if len(two_divs) != 2:
            logging.error(f"not 2 divs: {two_divs}")
            continue
        tag : bs4.element.Tag = two_divs[1] 
        
        attr = tag.get_text().strip().lower()
        logging.debug(f"find attr: {attr}")
        attrs.append(attr)

    return attrs

def get_battlesuit_data(wiki_address : str):
    battlesuit_page = requests.get(wiki_address)
    battlesuit_soup = BeautifulSoup(battlesuit_page.content, 'html.parser')
    battlesuit_soup.encode("utf-8")

    main_class = battlesuit_soup.find("main", class_="page__main")
    main_class = main_class.find("div", class_="dotted-background",
        style="display: flex; width: fit-content; padding-top: 2px; padding-bottom: 2px; padding-left: 2px; border-radius: 10px; justify-content: flex-start; box-shadow: 0 0 2px rgb(0 0 0 / 50%); margin: 2px; outline: 1px solid rgb(0 0 0 / 20%); background-color: rgb(63 113 152 / 20%);"
    )
    
    rank = get_bs_rank(main_class)
    battlesuit_name, img_src = get_bs_img(main_class)
    
    profile_box = main_class.find("div",
        style="display: flex; flex-direction: column; align-content: flex-start; justify-content: flex-start; align-items: flex-start;"
    )

    bs_type = get_bs_type(profile_box)

    bs_version = get_bs_release_version(profile_box)

    bs_tags = get_bs_core_strengths(profile_box)

    return {
        "name" : battlesuit_name,
        "img" : img_src,
        "type" : bs_type,
        "version" : bs_version,
        "tags" : bs_tags,
        "rank" : rank,
    }


def parse_battlesuit(tr : bs4.element.Tag, name : str):
    # get all a
    battlesuits = []
    a_list = tr.find_all("a")
    for a in a_list:
        href = a.get("href")
        battlesuit_data = get_battlesuit_data(BASE_URL+href)
        battlesuits.append(battlesuit_data)
    return battlesuits

def parse_character(trs, path):

    for tr in trs:
        
        tr : bs4.element.Tag
        first_td : bs4.element.Tag = tr.find("td")
        second_td = first_td.find_next_sibling("td")
        name = parse_base_character(first_td)
        battlsuit_ch = parse_battlesuit(second_td, name)
        data[name] = {
            "name" : name,
            "battlesuit" : battlsuit_ch,
        }
        print()

def parse(path : str):
    page = requests.get(URL)
    #encode utf-8
    sleep(1)
    soup = BeautifulSoup(page.content, 'html.parser')
    soup.encode("utf-8")

    # 
    raw_focus = soup.find(
        "table",
        class_="navbox infobox-base",
        style="width: 100%; vertical-align: left; margin-top: 10px;;"
    )

    raw_focus_1 = raw_focus.find("tr")
    raw_focus_2 = raw_focus_1.find("td")
    raw_focus_3 = raw_focus_2.find("table")

    trs = raw_focus_3.find_all("tr")[1:]
    # remove all tr that has style height:2px;
    selected = []
    for tr in trs:
        if tr.get("style") is not None:
            logging.debug("removing tr with style: %s", tr.get("style"))
            continue
        selected.append(tr)

    del page
    del soup
    del raw_focus
    del raw_focus_1
    del raw_focus_2
    del raw_focus_3

    logging.debug(f"characters: {len(selected)}")
    parse_character(selected, path)

    with open(path, "w") as f:
        json.dump(data, f, indent=4)

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)

    parse(os.path.join(HONKAIDEX_DATA, f"{script_name}.json"))


from honkaiDex import gconfig

def valid_na_uid(uid : str) -> bool:
    """
    Check if the uid is valid.
    """
    try:
        uid_str = str(uid)
        uid_int = int(uid)
    except:
        return False

    if not uid_str.isdigit():
        return False

    if len(uid_str) != 9:
        return False

    if uid_str[0] != "1":
        return False

    return True

def valid_lv(lv : str) -> bool:
    """
    Check if the lv is valid.
    """
    try:
        int_lv = int(lv)
    except:
        return False

    if lv is None or lv == "":
        return False
    
    minlv = gconfig.MIN_LV
    maxlv = gconfig.MAX_LV

    if minlv <= int_lv <= maxlv:
        return True
    return False


import logging
import os
import json

HONKAIDEX_DATA = os.path.dirname(os.path.realpath(__file__))

__LOADED_TRACK__ = {}

def load_once():
    # wrapper
    def load_once_wrapper(func):
        def wrapper(*args, **kwargs):
            func_module = str(func.__module__)

            if func_module+func.__name__ in __LOADED_TRACK__:
                # get func module name

                logging.debug(f"{func_module}.{func.__name__} already loaded")
                return
            __LOADED_TRACK__[func_module+func.__name__] = True
            logging.debug(f"{func_module}.{func.__name__} loaded")
            return func(*args, **kwargs)
        return wrapper

    return load_once_wrapper

# load function
def load(load_str : str):
    # based on str find .py file within package
    # load_str = "honkaiDex.data.stigamata.stigamata_1"
    import importlib
    logging.debug(f"loading module {load_str}")
    mod = importlib.import_module("honkaiDex.data."+load_str)
    if not hasattr(mod, "load"):
        logging.debug(f"{load_str} has no load function")

    mod.load()



from setuptools import setup, find_packages
import os

# find all json files in honkaiDex/data/
data_files = []
for root, dirs, files in os.walk("honkaiDex/data"):
    for file in files:
        if file.endswith(".json"):
            data_files.append(os.path.join(root, file))

setup(
    name="honkaidex",
    version="0.2.0",
    author="celtica, kiyandere",
    author_email="celticaxp@gmail.com, kiyanhalcyon0707@gmail.com",
    description="Honkai Impact 3 Database",
    long_description="".join(open("README.md", "r").readlines()),
    long_description_content_type="text/markdown",
    url="https://github.com/HiganHana/HonkaiDex",
    packages=[
        "honkaiDex",
        "honkaiDex.base",
        "honkaiDex.profile",
        "honkaiDex.data",
        "honkaiDex.parse_v1",
        "honkaiDex.game",
    ],
    data_files=data_files,
    install_requires=[
        "zw-util-lib",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True
)